/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef CLIENT_SETTINGS_INC
#define CLIENT_SETTINGS_INC

#include "base/util/StringBuffer.h"
#include "client/DMTClientConfig.h"
#include "Funambol.hrh"
#include "commonDefs.h"
#ifdef __GCCE__ 
#include "SyncEngine.h"
#else
#include "FakeSyncEngine.h"
#endif

using namespace Funambol;

class ClientSettings : public DMTClientConfig
{
public:
    static ClientSettings* getInstance();

    virtual bool save();
    virtual bool read();

    /**
     * Loads the ConfigSyncSource settings (meaning: handled by the config sync source)
     * @return  true if no errors
     */
    bool readConfigSyncSourceSettings();

    /**
     * Called the first time the client runs.
     * The config is not found, so we generate it at runtime.
     */
    void createDefaultConfig();
    
    /**
     * Called every time the Client starts, to check if a new
     * version has been installed. If true, we execute proper actions
     * here to upgrade the client configuration.
     * When installing a new version, we only copy the file '\upgrade\config.ini'
     * with all necessary information - here we check that file to really
     * upgrade the client at runtime.
     * 
     * @return true if the configuration has been upgraded to a new version
     */
    bool makeUpgradeActions();
    
    /**
     * Creates the SyncML profile with name 'SYNC_PROFILE_NAME' and
     * propagates the current configuration settings into the profile.
     * @return  true if profile created without errors
     */
    bool createSyncMLProfile();
    
    /**
     * Creates the IMAP account with name 'SYNC_PROFILE_NAME' and
     * propagates the current configuration settings into the account.
     * If the account already exists, it just updates the account settings
     * and returns false.
     * @return  true if account created without errors
     */
    bool createIMAPAccount();
    
    /**
     * Utility method to check if credentials are empty.
     * We check username, password and syncURL values.
     * @return true if at least one credential field is empty.
     */
    bool isCredentialsEmpty();
    
    /**
     * Checks if we copied the account data from existing SyncML profile.
     * Usually false, it may happen if it's the first execution of the Client and
     * a profile named SYNC_PROFILE_NAME is already existing (since 7.1.12).
     */
    bool checkCopiedAccountFromProfile();

    
    // ---- get methods ----
    const StringBuffer&   getDisplayName()             { return displayName;       }
    const StringBuffer&   getEmailAddress()            { return emailAddress;      }
    const TSyncUsing      getSyncUsing()               { return syncUsing;         }
    const bool            getPendingSyncConfig()       { return pendingSyncConfig; }
    const TInt            getSchedulerMinutes()        { return schedulerMinutes;  }
    const TInt            getCpInterval()              { return cpInterval;        }
    const StringBuffer&   getIapName()                 { return iapName;           }
    const TInt64          getLastEmailSyncTimestamp()  { return lastEmailSyncTimestamp;  }
    const TLastSyncStatus getLastEmailSyncStatus()     { return lastEmailSyncStatus;     }
    const TInt            getPushType()                { return pushType;                }
    
    const TInt64          getFirstMediaSyncTimestamp(const TInt aSourceID);
    const TInt64          getFirstPicturesSyncTimestamp() { return firstPicturesSyncTimestamp;}
    const TInt64          getFirstVideosSyncTimestamp()   { return firstVideosSyncTimestamp;  }
    
    const TInt64          getInstallTimestamp()        { return installTimestamp;        }
    
    const bool            getIncludeOlderMedia(const TInt aSourceID);
    const bool            getIncludeOlderPictures()    { return includeOlderPictures;    }
    const bool            getIncludeOlderVideos()      { return includeOlderVideos;      }
    const TLastSyncStatus getLastSyncStatus(const TInt aSourceID);
    const TInt            getMediaStorage(const TInt aSourceID);
    const TInt            getPicturesStorage()         { return pictureStorageType;      }
    const TInt            getVideosStorage()           { return videoStorageType;        }
    const StringBuffer&   getOldSwVersion()            { return oldSwVersion;            } 
    
    
    // ---- set methods ----
    void setDisplayName             (const StringBuffer& aDisplayName);
    void setEmailAddress            (const StringBuffer& aEmailAddress);
    void setPendingSyncConfig       (const bool aPendingSyncConfig);
    void setSyncUsing               (const TSyncUsing aSyncUsing);
    void setLastEmailSyncTimestamp  (TInt64 aTime);
    void setLastEmailSyncStatus     (TLastSyncStatus aStatus);
    void setFirstMediaSyncTimestamp   (const TInt aSourceID, TInt64 aTime, const bool storeNow = true);
    void setFirstPicturesSyncTimestamp(TInt64 aTime, const bool storeNow = true);
    void setFirstVideosSyncTimestamp  (TInt64 aTime, const bool storeNow = true);
    void setInstallTimestamp        (TInt64 aTime);
    
    void setIncludeOlderMedia       (const TInt aSourceID, const bool aInclude);
    void setIncludeOlderPictures    (const bool aInclude);
    void setIncludeOlderVideos      (const bool aInclude);
    void setLastSyncStatus          (const TInt aSourceID, TLastSyncStatus aStatus);
    void setSchedulerMinutes        (const TInt aSchedulerMinutes);
    void setCpInterval              (const TInt aCpInterval);
    void setIapName                 (const StringBuffer& aIapName);
    void setImapAccountLuid         (const StringBuffer& aLuid);
    void setPassword                (const StringBuffer& aPassword);
    void setSyncURL                 (const StringBuffer& aSyncURL);
    void setUsername                (const StringBuffer& aUsername);
    void setMediaStorage            (const TInt aSourceID, const TInt aStorage);
    void setPicturesStorage         (const TInt aPicturesStorage);
    void setVideosStorage           (const TInt aVideosStorage);
    void setClientSwVersion         (const StringBuffer& aClientSwVersion);
    void setOldSwVersion            (const StringBuffer& aSwVersion);
    void setPushType                (const TInt aPushType);
    void forceDirtyOnMainNode();

    
    /// Returns the ArrayList of sources visible.
    const ArrayList& getSourcesVisible();
    
    /**
     * Adds the passed source name to the sourcesVisible array.
     * The element is appended at the end.
     */
    bool safeAddSourceVisible(const char* sourceName);
    
    /**
     * Adds the passed source name to the sourcesVisible array, in position 'pos'.
     * - the source is not added if already exists in the array
     * - if the source is already found in a different position, it is moved
     * @param sourceName  the source name to add
     * @param pos         the position to insert in the sourcesVisible array
     * @return true       if the element is added, false if not found
     */
    bool safeAddSourceVisible(const char* sourceName, const int pos);

    /**
     * Removes the passed source name from the sourcesVisible array.
     * @return true if the source was found and removed, false if not found
     */
    bool removeSourceVisible(const char* sourceName);

    /**
     * Reads the 'sourceOrder' from config file and populates the sourcesVisible array.
     * The 'sourceOrder' value is a comma separated string of source names.
     * @note  contacts,calendar&tasks cannot be hidden for now
     */
    void readSourcesVisible();

    /**
     * Reads the sourcesVisible array and saves the 'sourceOrder' config file.
     * The 'sourceOrder' value is a comma separated string of source names.
     */
    void saveSourcesVisible();  
    
    /**
     * When invoked, changes the sync mode for the selected source.
     * For example: two-way -> one-way-client -> disabled -> two-way ...
     * Only the available modes for the desired source are used.
     * The config is saved at the end.
     */
    void rotateSyncMode(const char* sourceName);
    
    /// Sets the flag "oneWayRemoval"
    void setOneWayRemoval(const bool val) { oneWayRemoval = val; }
    
    /// Gets the "oneWayRemoval" flag
    bool getOneWayRemoval() { return oneWayRemoval; }
    
    
protected:
    
    /**
     * Overrides SyncManagerConfig::addSyncSourceConfig(&sc).
     * Reimplemented to consider the mail syncsource config as a normal 
     * config (not a MailSyncSourceConfig).
     * 
     * Add the passed SyncSourceConfig (the object is copied).
     * If SyncSourceConfig name is already present (match name) the
     * config is replaced with the given one.
     * Otherwise it is added in the sourceConfig array.
     */
    bool addSyncSourceConfig(SyncSourceConfig& sc);
    
    /**
     * Overrides DMTClientConfig::saveSourceConfig().
     * Reimplemented to consider the mail syncsource config as a normal 
     * config (not a MailSyncSourceConfig).
     */
    void saveSourceConfig(int i, ManagementNode& n);
    
private:
    
    ClientSettings();
    
    /**
     * Creates and return a StringBuffer with the deviceID: "IMEI:xxxxxxxx".
     */
    StringBuffer retrieveDeviceId();
    
    StringBuffer getImeiL();

    /**
     * Returns a property value from its name, reading from the upgrade dir.
     * Upgrade dir is under 'APPLICATION_URI/upgrade' path.
     */
    StringBuffer getPropertyFromUpgradeDir(const StringBuffer& propertyName);
    
    /// Formats and return a UserAgent string, based on the sw version.
    /// In the format "<application name> v. x.y.z".
    StringBuffer createUserAgent(const StringBuffer& swv);
    
    // --- Not used: dirs are created automatically by DMNode if fopen fails ---
    //TInt createAllDirs(const StringBuffer& rootDir, RArray<TInt>& sourceIDs);
    //TInt makeDir(const StringBuffer& aPath, const char* aContext, RFs* aFileSession);

    /**
     * Resets and fills the sourceVisible array reading default values, from customization.h.
     * It's used to generate default config (1st run) or in case of upgrade from old versions.
     * @return  the number of sources visible
     */
    int createDefaultSourceVisible();
    
    
private:
    static ClientSettings* instance;
    static const StringBuffer configContext;
    static const StringBuffer upgradeContext;
    static const StringBuffer addressNodeName;
    static const StringBuffer displayNodeName;
    static const StringBuffer nodePropertyName;
    static const StringBuffer pendingSyncConfigPropertyName;
    static const StringBuffer syncUsingPropertyName;
    static const StringBuffer lastEmailSyncTimestampPropertyName;
    static const StringBuffer lastEmailSyncStatusPropertyName;
    static const StringBuffer firstPicturesSyncTimestampPropertyName;
    static const StringBuffer firstVideosSyncTimestampPropertyName;
    static const StringBuffer installTimestampPropertyName;
    static const StringBuffer includeOlderPicturesPropertyName;
    static const StringBuffer includeOlderVideosPropertyName;
    
    static const StringBuffer lastContactsSyncStatusPropertyName;
    static const StringBuffer lastCalendarSyncStatusPropertyName;
    static const StringBuffer lastNotesSyncStatusPropertyName;
    static const StringBuffer lastConfigSyncStatusPropertyName;
    static const StringBuffer lastPhotosSyncStatusPropertyName;
    static const StringBuffer lastVideosSyncStatusPropertyName;
    
    static const StringBuffer schedulerMinutesPropertyName;
    static const StringBuffer cpIntervalPropertyName;
    static const StringBuffer iapNamePropertyName;
    static const StringBuffer clientSwVersionPropertyName;
    static const StringBuffer pictureStorageTypePropertyName;
    static const StringBuffer videoStorageTypePropertyName;
    static const StringBuffer oldSwVersionPropertyName;
    static const StringBuffer pushTypePropertyName;
    static const StringBuffer sourceOrderPropertyName;
    
    /**
     * The list of sources visible in the Client's UI. Sources not listed
     * here are hidden to the user.
     */
    ArrayList sourcesVisible;

    StringBuffer    emailAddress;
    StringBuffer    displayName;
    bool            pendingSyncConfig;
    TSyncUsing      syncUsing;
    TInt64          lastEmailSyncTimestamp;
    TLastSyncStatus lastEmailSyncStatus;
    TInt64          firstPicturesSyncTimestamp;     /**< It's the timestamp of the very first pictures sync  */   
    TInt64          firstVideosSyncTimestamp;       /**< It's the timestamp of the very first videos sync    */
    TInt64          installTimestamp;               /**< It's the timestamp of the installation of client v8.5 or later (introduced in v.8.5.0) */
    bool            includeOlderPictures;           /**< If false, all pictures before installTimestamp will be filtered out (default = false) */
    bool            includeOlderVideos;             /**< If false, all pictures before installTimestamp will be filtered out (default = false) */
    
    TLastSyncStatus lastContactsSyncStatus;
    TLastSyncStatus lastCalendarSyncStatus;
    TLastSyncStatus lastNotesSyncStatus;
    TLastSyncStatus lastConfigSyncStatus;
    TLastSyncStatus lastPhotosSyncStatus;
    TLastSyncStatus lastVideosSyncStatus;
    
    TInt            schedulerMinutes;
    TInt            cpInterval;        
    StringBuffer    iapName;
    bool            dirty;                          /**< Is the main node dirty?                                    */
    bool            nativeDirty;                    /**< Are native profiles dirty?                                 */
    bool            copiedAccountFromProfile;       /**< If we copied the account data from existing SyncML profile */
    StringBuffer    clientSwVersion;                /**< client sw version (distinguished from svw)                 */
    TInt            pushType;                       /**< Bit-mask to specify the pushtype (see PushEngine.h)        */
    TInt            pictureStorageType;             /**< Drive type where pictures are stored (main memory, mass storage or memory card) */
    TInt            videoStorageType;               /**< Drive type where videos are stored (main memory, mass storage or memory card) */
    StringBuffer    oldSwVersion;                   /**< sw version saved after client upgrade                      */

    /// If true, the one-way syncmode was removed upon upgrade 
    /// and there was at least one source with a one-way syncmode set.
    /// It will cause a warning popup + the source is disabled + next sync is slow.
    bool            oneWayRemoval;
};

#endif

