/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef __FUNAMBOLAPPVIEW_h__
#define __FUNAMBOLAPPVIEW_h__


#include <coecntrl.h>
#include <aknsbasicbackgroundcontrolcontext.h>
#include "FunambolAppUi.h"
#include "ListboxItem.h"

#include <aknlists.h> 	// For List Box
#include <barsread.h> 	// For Resource Reader
#include <eikclbd.h>    // For iListBox->ItemDrawer() !



/**
 * The FunambolAppView class represents the viewer of the main screen. 
 * It has methods to draw and refresh the graphical  object on the main screen.
 * It contains a listbox: a list of graphical elements (1 icon + text + small text),
 *  each of them is called pane and represents a single source to be synchronized.
 *  The FunambolAppView is able to load, update and dynamically modify the objects on the UI, 
 *  as well as get informations about the status of these elements. 
 *  This class also manages the keyboard's input, moving across the listbox when the 
 *  up/down arrows are pressed or taking the right action when the fire button is clicked.
 */
class CFunambolAppView : public CCoeControl
{

public:

    /**
    * Two-phased constructor.
    * Create a CFunambolAppView object, which will draw itself to aRect.
    * @param aRect The rectangle this view will be drawn to.
    * @return a pointer to the created instance of CFunambolAppView.
    */
    static CFunambolAppView* NewL(const TRect& aRect, CFunambolAppUi* aAppUI);

    /**
    * Two-phased constructor.
    * Create a CFunambolAppView object, which will draw itself
    * to aRect.
    * @param aRect Rectangle this view will be drawn to.
    * @return A pointer to the created instance of CFunambolAppView.
    */
    static CFunambolAppView* NewLC(const TRect& aRect, CFunambolAppUi* aAppUI);

    /**
    * Virtual Destructor.
    */
    virtual ~CFunambolAppView();
    
    /**
     * Refreshes all the UI screen.
     * Reads sourceOrder array from the config, and updates the iListboxItems array.
     * Then updates the UI iListbox as well.
     * 
     * To show/hide panes, just update the sourceOrder array in config and then call
     * this method.
     */
    void refresh();
    
    /// Refresh the status text of the source, with the given text.
    void RefreshSourceStatus(const TInt aSourceID,  const TDesC& aText, const TBool aDimmed = EFalse);

    /// Refresh the status text of the source, with the buffered status text (the last one).
    void RefreshSourceBufferedStatus(const TInt aSourceID,  const TBool aDimmed = EFalse);


    bool isSourceDisplayed(const TInt aSourceID);

    TInt GetCurrentItemSelected();
    TInt GetCurrentSourceSelected();
    
    /// Scrolls the listbox to make the desired source pane visible.
    void scrollToSourcePane(const TInt aSourceID);
    

private:

    CFunambolAppUi*                     iAppUI;

    /// The graphical listbox (icon + label + smaller text).
    CEikFormattedCellListBoxTypedef*    iListBox;

    /**
     * Array of ListboxItem objects, each one is a row of the listbox.
     * Each item contains all properties of a single pane.
     * It is dynamically managed: modify this array and then call the
     * method 'refresh()' in order to update the UI screen accordingly.
     */ 
    ArrayList                           iListboxItems;

    /**
     * Array of source names (StringBuffer) of all the sources that may have a UI.
     * It includes sources that are visible or not, indifferently.
     * It is initialized only once at startup (used just to load all icons only once)
     * @see SOURCES_WITH_UI define
     */
    ArrayList                           allSourcesWithUI;

    /// Used to avoid the cuncurrent refresh of Listbox items, from different threads.
    RCriticalSection                    iCriticalListboxText;

    CAknsBasicBackgroundControlContext* iBgContext;


    /**
    * 2nd phase constructor.
    * Perform the second phase construction of a
    * CFunambolAppView object.
    * @param aRect The rectangle this view will be drawn to.
    */
    void ConstructL(const TRect& aRect, CFunambolAppUi* aAppUI);

    /**
    * C++ default constructor.
    */
    CFunambolAppView();

    /**
     * Creates the UI listbox.
     * The listbox is loaded empty at startup, before the config is read 
     * (so the UI is displayed quicky at startup).
     * Then the desired source panes will be added dynamically, calling
     * method 'refresh()'.
     */
    void CreateListBoxL(const TRect&/* aRect*/);

    /**
     * Updates all the UI panes.
     * This method reads the iListboxItems ArrayList and persists
     * its data to the device screen.
     */
    void refreshAllPanes();

    /**
     * Updates the UI pane corresponding to the item passed.
     * @param item       the ListboxItem to read from
     * @param itemIndex  the position of the UI pane to refresh with the new data
     */
    void refreshPane(ListboxItem* item, const int itemIndex);
    
    /**
     * Returns the ListboxItem corresponding to the source ID passed.
     * Returns NULL if not found.
     */
    ListboxItem* getListboxItem(const TInt aSourceID);
    
    /**
     * Returns the ListboxItem corresponding to the source ID passed.
     * Returns NULL if not found.
     * @param index [OUT] the position of the source pane in the list if found 
     *                    KErrNotFound (-1) if not found
     */
    ListboxItem* getListboxItem(const TInt aSourceID, int* index);


    /// Returns the number of source panes currently displayed.
    int countSourcesDisplayed();

    /**
     * Returns the icon's index corresponding to the source ID passed.
     * The index returned is the one corresponding to the normal (solid) icon.
     * All icons are set in the ListBox's icons array at startup, and
     * have a fixed index.
     */
    int getIconIndex(const TInt aSourceID);
    
    
    // --------- Functions from base classes -------------
    /**
    * From CCoeControl, Draw
    * Draw this CFunambolAppView to the screen.
    * @param aRect the rectangle of this view that needs updating
    */
    void Draw( const TRect& aRect ) const;

    /**
    * From CoeControl, SizeChanged.
    * Called by framework when the view size is changed.
    */
    virtual void SizeChanged();

    TTypeUid::Ptr MopSupplyObject(TTypeUid aId);

    /// Overrided to capture the key events: up/down arrow, selection key.
    TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);

    /// Returns number of controls inside this container, now = 1.
    TInt CountComponentControls() const;

    /// Returns the internal control given its index.
    CCoeControl* ComponentControl(TInt aIndex) const;
    // -----------------------------------------------------
    
};

#endif // __FUNAMBOLAPPVIEW_h__
